/* Copyright (C) 2011, 2012, 2013 by Bundesanstalt für Gewässerkunde
 * Software engineering by Intevation GmbH
 *
 * This file is Free Software under the GNU AGPL (>=v3)
 * and comes with ABSOLUTELY NO WARRANTY! Check out the
 * documentation coming with Dive4Elements River for details.
 */

package org.dive4elements.river.exports.minfo;

import org.apache.log4j.Logger;
import org.jfree.data.xy.XYSeries;
import org.w3c.dom.Document;

import org.dive4elements.artifactdatabase.state.ArtifactAndFacet;
import org.dive4elements.artifactdatabase.state.Facet;
import org.dive4elements.river.artifacts.model.FacetTypes;
import org.dive4elements.river.artifacts.model.minfo.BedDiffYearResult;
import org.dive4elements.river.artifacts.model.minfo.BedDifferencesResult;
import org.dive4elements.river.exports.StyledSeriesBuilder;
import org.dive4elements.river.exports.fixings.FixChartGenerator;
import org.dive4elements.river.exports.process.KMIndexProcessor;
import org.dive4elements.river.exports.process.Processor;
import org.dive4elements.river.jfree.Bounds;
import org.dive4elements.river.jfree.DoubleBounds;
import org.dive4elements.river.jfree.RiverAnnotation;
import org.dive4elements.river.jfree.StyledXYSeries;


public class BedDiffHeightYearGenerator
extends FixChartGenerator
implements FacetTypes
{
    public enum YAXIS {
        D(0), dW(1);

        protected int idx;

        private YAXIS(int c) {
            idx = c;
        }
    }

    /** The logger that is used in this generator. */
    private static Logger logger = Logger.getLogger(BedDiffHeightYearGenerator.class);

    public static final String I18N_CHART_TITLE = "chart.beddifference.height.title";
    public static final String I18N_XAXIS_LABEL = "chart.beddifference.height.xaxis.label";
    public static final String I18N_YAXIS_LABEL = "chart.beddifference.height.yaxis.label";

    public static final String I18N_CHART_TITLE_DEFAULT = "Sohlenhöhen Differenz";
    public static final String I18N_XAXIS_LABEL_DEFAULT = "Fluss-Km";
    public static final String I18N_YAXIS_LABEL_DEFAULT = "delta S [cm / Jahr]";
    public static final String I18N_DW_YAXIS_LABEL_DEFAULT  =
            "delta W [cm]";
    public static final String I18N_DW_YAXIS_LABEL =
            "chart.fixings.longitudinalsection.yaxis.label";

    @Override
    protected YAxisWalker getYAxisWalker() {
        return new YAxisWalker() {

            @Override
            public int length() {
                return YAXIS.values().length;
            }

            @Override
            public String getId(int idx) {
                YAXIS[] yaxes = YAXIS.values();
                return yaxes[idx].toString();
            }
        };
    }

    @Override
    public void doOut(ArtifactAndFacet bundle, Document attr, boolean visible) {
        String name = bundle.getFacetName();

        logger.debug("doOut: " + name);

        if (name == null) {
            logger.error("No facet name for doOut(). No output generated!");
            return;
        }

        Facet facet = bundle.getFacet();

        if (facet == null) {
            return;
        }

        if (bundle.getData(context) instanceof BedDifferencesResult) {
            setContextBounds(bundle);
        }

        Processor processor = new KMIndexProcessor();
        if (name.equals(BED_DIFFERENCE_HEIGHT_YEAR)) {
            doBedDifferenceYearOut(
                (BedDiffYearResult) bundle.getData(context),
                bundle, attr, visible);
        }
        else if (name.equals(BED_DIFFERENCE_HEIGHT_YEAR_FILTERED)) {
            doBedDifferenceYearOut(
                (BedDiffYearResult) bundle.getData(context),
                bundle, attr, visible);
        }
        else if (name.equals(LONGITUDINAL_ANNOTATION)) {
            doAnnotations(
                (RiverAnnotation) bundle.getData(context),
                 bundle,
                 attr,
                 visible);
        }
        else if (processor.canHandle(name)) {
            processor.doOut(this, bundle, attr, visible, YAXIS.dW.idx);
        }
        else {
            logger.warn("Unknown facet name " + name);
        }
    }

    public void setContextBounds(ArtifactAndFacet bundle) {
        if (getXBounds(0) != null && getDomainAxisRange() != null) {
            Bounds bounds =
                calculateZoom(getXBounds(0), getDomainAxisRange());
            context.putContextValue("startkm", bounds.getLower());
            context.putContextValue("endkm", bounds.getUpper());
        }
        else if (getXBounds(0) != null && getDomainAxisRange() == null) {
            context.putContextValue("startkm", getXBounds(0).getLower());
            context.putContextValue("endkm", getXBounds(0).getUpper());
        }
        else if (getXBounds(0) == null && getDomainAxisRange() == null) {
            BedDifferencesResult data = (BedDifferencesResult)bundle.getData(context);
            if (data.getKms().size() >= 0) {
                context.putContextValue("startkm", data.getKms().min());
                context.putContextValue("endkm", data.getKms().max());
            }
            else {
                logger.warn("No data to define start and end km");
            }
        }
        else if (getXBounds(0) == null && getDomainAxisRange() != null){
            BedDifferencesResult data = (BedDifferencesResult)bundle.getData(context);
            Bounds b = new DoubleBounds(data.getKms().min(), data.getKms().max());
            Bounds bounds =
                calculateZoom(b, getDomainAxisRange());
            context.putContextValue("startkm", bounds.getLower());
            context.putContextValue("endkm", bounds.getUpper());
        }
    }

    @Override
    protected String getDefaultChartTitle() {
        return msg(I18N_CHART_TITLE, I18N_CHART_TITLE_DEFAULT);
    }

    @Override
    protected String getDefaultXAxisLabel() {
        return msg(I18N_XAXIS_LABEL,
            I18N_XAXIS_LABEL_DEFAULT,
            new Object[] { getRiverName() });
    }

    @Override
    protected String getDefaultYAxisLabel(int pos) {
        if (pos == YAXIS.D.idx) {
            return msg(I18N_YAXIS_LABEL, I18N_YAXIS_LABEL_DEFAULT);
        }
        else if (pos == YAXIS.dW.idx) {
            return msg(I18N_DW_YAXIS_LABEL, I18N_DW_YAXIS_LABEL_DEFAULT);
        }
        return "default";
    }

    protected void doBedDifferenceYearOut(BedDiffYearResult data,
        ArtifactAndFacet aandf, Document theme, boolean visible) {

        XYSeries series = new StyledXYSeries(aandf.getFacetDescription(), theme);
        StyledSeriesBuilder.addPoints(series, data.getHeightPerYearData(), true);

        addAxisSeries(series, YAXIS.D.idx, visible);
    }
}
